#
# spec file for package containerd
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


# Check if go_arches is defined in the project configuration
# Otherwise, define it here
# In order to define it in the project configuration, see
#
# https://en.opensuse.org/openSUSE:Build%20Service%20prjconf#Macros
#
# The Macros tag is the one that defines the go_arches variable to be used
# in the spec file.
# The "define" one is to help the specfile parser of the buildservice
# to see what packages are being built. You also want to define it here
# for keeping things consistent.

%{!?go_arches: %global go_arches %ix86 x86_64 aarch64 ppc64le}

%ifarch %go_arches
%define go_tool go
%define GO_BUILD_FLAGS %{nil}
%else
%define go_tool go-6
%define GO_BUILD_FLAGS "-gccgoflags=-Wl,--add-needed -Wl,--no-as-needed -static-libgo -ldl"
%endif

%define git_version 973f21f
%define version_unconverted 0.2.3

Name:           containerd
Version:        0.2.3
Release:        3.1
Summary:        Standalone OCI Container Daemon
License:        Apache-2.0
Group:          System/Management
Url:            https://containerd.tools
Source:         %{name}-%{version}.tar.xz
Source1:        %{name}.service
Source2:        %{name}.socket
Source3:        sysconfig.%{name}
%ifarch %go_arches
BuildRequires:  go >= 1.5
%else
BuildRequires:  gcc6-go >= 6.1
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
# Currently runC is the only supported runtime for containerd. NOTE:
# This version pinning is done to follow (as well as we can) upstream's
# Dockerfile.  However, for some reason the Dockerfile for containerd
# diverges from the one for Docker. As such, this cannot be a hard
# requirement (that would break Docker).
Requires:       runc = 0.1.1+gitcc29e3d

%define client ctr
%define daemon %{name}
%define shim %{name}-shim

%description
Containerd is a daemon with an API and a command line client, to manage
containers on one machine. It uses runC to run containers according to the OCI
specification. Containerd has advanced features such as seccomp and user
namespace support as well as checkpoint and restore for cloning and live
migration of containers.

%package ctr
Summary:        Client for %{name}
Group:          System/Management
Requires:       %{name} = %{version}
%ifarch %go_arches
BuildRequires:  go >= 1.5
%else
BuildRequires:  gcc6-go >= 6.1
%endif

%description ctr
Standalone client for containerd, which allows management of containerd containers
separately from Docker.

%package test
Summary:        Test package for containerd
Group:          System/Management
%ifarch %go_arches
Requires:       go >= 1.5
%else
Requires:       gcc5-go >= 5.0
%endif
BuildArch:      noarch

%description test
Test package for containerd. It contains the source code and the tests.

%prep
%setup -q

%build
# Do not use symlinks. If you want to run the unit tests for this package at
# some point during the build and you need to directly use go list directly it
# will get confused by symlinks.
export GOPATH=${HOME}/go:${HOME}/go/src/github.com/docker/containerd/vendor
mkdir -pv $HOME/go/src/github.com/docker/containerd
rm -rf $HOME/go/src/github.com/docker/containerd/*
cp -avr * $HOME/go/src/github.com/docker/containerd

# Build daemon.
%go_tool build %GO_BUILD_FLAGS -x -o %{daemon}-%{version} github.com/docker/%{name}/%{daemon}

# Build shim.
%go_tool build %GO_BUILD_FLAGS -x -o %{shim}-%{version} github.com/docker/%{name}/%{shim}

# Build client.
%go_tool build %GO_BUILD_FLAGS -x -o %{client}-%{version} github.com/docker/%{name}/%{client}

%ifarch %go_arches
%check
export GOPATH=$HOME/go/src/github.com/docker/containerd/vendor:$GOPATH
cd $HOME/go/src/github.com/docker/containerd
go test -v $(go list ./... | grep github.com/docker/containerd | grep -v /integration-test)
%endif

%install
%{__install} -D -m755 %{daemon}-%{version} %{buildroot}/%{_sbindir}/%{daemon}
%{__install} -D -m755 %{shim}-%{version} %{buildroot}/%{_sbindir}/%{shim}
%{__install} -D -m755 %{client}-%{version} %{buildroot}/%{_sbindir}/%{name}-%{client}

# Add service and sysconfig.
%{__install} -D -m644 %SOURCE1 %{buildroot}/%{_unitdir}/%{name}.service
%{__install} -D -m644 %SOURCE2 %{buildroot}/%{_unitdir}/%{name}.socket
ln -sf %{_sbindir}/service %{buildroot}/%{_sbindir}/rccontainerd
%{__install} -D -m644 %SOURCE3 %{buildroot}/var/adm/fillup-templates/sysconfig.%{name}

install -d -m755 %{buildroot}/usr/src/containerd/
cp -avr $HOME/go/src/github.com/docker/containerd/* %{buildroot}/usr/src/containerd/

%pre
%service_add_pre %{name}.service %{name}.socket

%post
%service_add_post %{name}.service %{name}.socket
%{fillup_only -n containerd}

%preun
%service_del_preun %{name}.service %{name}.socket

%postun
%service_del_postun %{name}.service %{name}.socket

%files
%defattr(-,root,root)
%doc LICENSE.code LICENSE.docs README.md
%{_sbindir}/%{daemon}
%{_sbindir}/%{shim}
%{_unitdir}/%{name}.service
%{_unitdir}/%{name}.socket
/var/adm/fillup-templates/sysconfig.%{name}
%{_sbindir}/rccontainerd

%files ctr
%{_sbindir}/%{name}-%{client}

%files test
%defattr(-,root,root)
/usr/src/containerd/
%exclude /usr/src/containerd/bin

%changelog
* Mon Sep 19 2016 jmassaguerpla@suse.com
- fix go_arches definition: use global instead of define, otherwise
  it fails to build
* Thu Aug 25 2016 jmassaguerpla@suse.com
- Remove GOPATH at the end of the GOPATH assignment
  cause GOPATH is empty and if we do that, we get the path ""
  appended, which causes gcc6-go to complain
* Wed Aug 24 2016 jmassaguerpla@suse.com
- add go_arches in project configuration: this way, we can use the
  same spec file but decide in the project configuration if to
  use gc-go or gcc-go for some archs.
* Mon Aug 22 2016 jmassaguerpla@suse.com
- update to v2.3.0 (bsc#995058)
- Remove patches which were already merged upstream:
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
* Thu Aug 18 2016 jmassaguerpla@suse.com
- use gcc6-go instead of gcc5-go (bsc#988408)
- build ppc64le with gc-go because this version builds with gc-go 1.6
* Thu Aug 18 2016 cbrauner@suse.com
- bump git commit id to the one required by docker v1.12.0
- run test during build
- only run tests on architectures that provide the go list and got test tools
- add aarch64 to go arches
* Thu Aug 18 2016 cbrauner@suse.de
- Add containerd-test package which contains the source code and the test. This
  package will be used to run the integration tests.
- Simplify package build and check sections: Instead of symlinking we default to
  cp -avr. go list gets confused by symlinks hence, we need to copy the source
  code anyway if we want to run unit tests during package build at some point.
* Tue Aug 16 2016 asarai@suse.com
  * Explicitly state the version dependency for runC, to avoid potential
  issues with incompatible component versions. These must be updated
  * each time we do a release*. Unfortunately we cannot create a hard
  dependency because that would conflict with Docker, and was a mistake
  on upstream's part. bsc#993847
* Wed Jul 20 2016 asarai@suse.com
  * Set --runtime option specifically to runC. bsc#978260
* Fri Jun 17 2016 asarai@suse.de
  * Update to containerd v0.2.2. (bsc#989566 FATE#320763)
  * Includes updates to the out-of-tree patches.
* Mon May  2 2016 asarai@suse.de
  * Remove MountFlags=slave from containerd.service. This causes many issues with
  interactions with Docker.
* Fri Apr 29 2016 asarai@suse.de
  * Added /usr/sbin/rccontainerd symlink as per suse-missing-rclink.
  * Updated socket activation patches to use the same patchset that was merged
  upstream (https://github.com/docker/containerd/pull/178):
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
  * Removed aarch64 that was patched upstream:
  - fix-aarch64-epoll.patch
  * Update containerd to 0.2.1. Upstream changelog:
  * Fixes for cgroup memory updates and process labeling.
  * Truncate the event log on disk and in memory so that it does not
    grow forever.  This is mainly used for higher levels to receive past
    events if they miss any.
* Tue Apr 19 2016 asarai@suse.de
  * Use the gc compiler for aarch64 builds.
  * Add a patch to fix the new aarch64 build support, which has not yet been
  merged upstream (https://github.com/docker/containerd/pull/195):
  + fix-aarch64-epoll.patch
  * Rebase the socket activation patchset which has yet to be merged
  (https://github.com/docker/containerd/pull/178):
  * socket-activation-01-vendor.patch
  * socket-activation-02-daemon.patch
  * socket-activation-03-ctr.patch
  * Update to containerd 0.2.0. Changelog:
  + Add Limit to PidsStats
  + Add timeout flag for container start times.
  + Add timeout option for GRPC connection.
  + Add no_pivot_root support.
  + Add runtimeArgs to pass to shim
  * Move epoll syscall to a separate package so we can build on aarch64.
  * Fix ctr termios restoration isssues.
  * Several bug fixes.
  - Remove dependencies on larger packages.
* Thu Mar 31 2016 asarai@suse.de
  * Use socket activation with the containerd-daemon. This requires a
  not-yet-upstream patchset (https://github.com/docker/containerd/pull/178):
  + socket-activation-01-vendor.patch
  + socket-activation-02-daemon.patch
  + socket-activation-03-ctr.patch
  * Remove MountFlags=slave since it's not relevant to containerd and might cause
  issues in the future.
  * Update to containerd 0.1.0. This required quite a few fixes.
* Thu Mar 31 2016 asarai@suse.de
  * Update to containerd 0.1.0. This required quite a few fixes.
  * Use socket activation with the containerd daemon. This requires a
  not-yet-upstream patchset (https://github.com/docker/containerd/pull/178):
  + socket-activation-01-vendor.patch
  + socket-activation-02-daemon.patch
  + socket-activation-03-ctr.patch
  * Remove MountFlags=slave since it's not relevant to containerd and might cause
  issues in the future.
* Sun Mar 27 2016 asarai@suse.de
  * Add initial packaging of containerd 0.0.5.
  * Add service and sysconfig files.
  * Separately package the client from the server.
  * Install to /usr/sbin.
* Sun Mar 27 2016 asarai@suse.de
  * Add initial packaging of containerd 0.0.5.
  * Install to /usr/sbin.
  * Separately package the client from the server.
